export declare type Fun<a, b> = (_: a) => b;
export declare type ListUpperBound<a, b> = b extends a ? b : a extends b ? a : never;
export declare type ListLowerBound<a, b> = any extends a ? never : a extends b ? never : a;
export declare type Pattern<a> = a extends number ? a | NumberConstructor : a extends string ? a | StringConstructor : a extends boolean ? a | BooleanConstructor : a extends Array<infer aa> ? [Pattern<aa>] : {
    [k in keyof a]?: Pattern<a[k]>;
};
export declare type InvertPattern<p> = p extends NumberConstructor ? number : p extends StringConstructor ? string : p extends BooleanConstructor ? boolean : p extends Array<infer pp> ? InvertPattern<pp>[] : {
    [k in keyof p]: InvertPattern<p[k]>;
};
/**
 * Constructs the builder API
 * @param value       the input value
 * @param otherwise   optional - a function that creates the fallback value
 * @param patterns    optional - an array with the patterns
 */
export declare const match: <a, b>(value: a) => {
    /**
     * Adds a pattern
     * @param pattern   the pattern to test with
     * @param expr      the function to create the result with
     */
    with: <p extends Pattern<a>>(pattern: p, expr: Fun<ListUpperBound<a, InvertPattern<p>>, b>) => {
        with: <p_1 extends Pattern<a>>(pattern: p_1, expr: Fun<ListUpperBound<a, InvertPattern<p_1>>, unknown>) => any;
        /**
         * Adds a pattern with an additional predicate
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withWhen: <p_2 extends Pattern<a>>(pattern: p_2, when: Fun<ListUpperBound<a, InvertPattern<p_2>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_2>>, unknown>) => any;
        /**
         * Adds a negated pattern. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        withNot: <p_3 extends Pattern<a>>(pattern: p_3, expr: Fun<ListLowerBound<a, InvertPattern<p_3>>, unknown>) => any;
        /**
         * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withNotWhen: <p_4 extends Pattern<a>>(pattern: p_4, when: Fun<ListLowerBound<a, InvertPattern<p_4>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_4>>, unknown>) => any;
        /**
         * Sets the faalback value for when no pattern matches
         * @param otherwise   a function to create the result
         */
        otherwise: (otherwise: () => unknown) => any;
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: () => unknown;
    };
    /**
     * Adds a pattern with an additional predicate
     * @param pattern   the pattern to test with
     * @param when      a predicate that has to be true, outside the mathing of the pattern
     * @param expr      the function to create the result with
     */
    withWhen: <p_5 extends Pattern<a>>(pattern: p_5, when: Fun<ListUpperBound<a, InvertPattern<p_5>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_5>>, b>) => {
        /**
         * Adds a pattern
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        with: <p_1 extends Pattern<a>>(pattern: p_1, expr: Fun<ListUpperBound<a, InvertPattern<p_1>>, unknown>) => any;
        withWhen: <p_2 extends Pattern<a>>(pattern: p_2, when: Fun<ListUpperBound<a, InvertPattern<p_2>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_2>>, unknown>) => any;
        /**
         * Adds a negated pattern. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        withNot: <p_3 extends Pattern<a>>(pattern: p_3, expr: Fun<ListLowerBound<a, InvertPattern<p_3>>, unknown>) => any;
        /**
         * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withNotWhen: <p_4 extends Pattern<a>>(pattern: p_4, when: Fun<ListLowerBound<a, InvertPattern<p_4>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_4>>, unknown>) => any;
        /**
         * Sets the faalback value for when no pattern matches
         * @param otherwise   a function to create the result
         */
        otherwise: (otherwise: () => unknown) => any;
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: () => unknown;
    };
    /**
     * Adds a negated pattern. Negated patterns don't work with `any` as input
     * @param pattern   the pattern to test with
     * @param expr      the function to create the result with
     */
    withNot: <p_6 extends Pattern<a>>(pattern: p_6, expr: Fun<ListLowerBound<a, InvertPattern<p_6>>, b>) => {
        /**
         * Adds a pattern
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        with: <p_1 extends Pattern<a>>(pattern: p_1, expr: Fun<ListUpperBound<a, InvertPattern<p_1>>, unknown>) => any;
        /**
         * Adds a pattern with an additional predicate
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withWhen: <p_2 extends Pattern<a>>(pattern: p_2, when: Fun<ListUpperBound<a, InvertPattern<p_2>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_2>>, unknown>) => any;
        withNot: <p_3 extends Pattern<a>>(pattern: p_3, expr: Fun<ListLowerBound<a, InvertPattern<p_3>>, unknown>) => any;
        /**
         * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withNotWhen: <p_4 extends Pattern<a>>(pattern: p_4, when: Fun<ListLowerBound<a, InvertPattern<p_4>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_4>>, unknown>) => any;
        /**
         * Sets the faalback value for when no pattern matches
         * @param otherwise   a function to create the result
         */
        otherwise: (otherwise: () => unknown) => any;
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: () => unknown;
    };
    /**
     * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
     * @param pattern   the pattern to test with
     * @param when      a predicate that has to be true, outside the mathing of the pattern
     * @param expr      the function to create the result with
     */
    withNotWhen: <p_7 extends Pattern<a>>(pattern: p_7, when: Fun<ListLowerBound<a, InvertPattern<p_7>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_7>>, b>) => {
        /**
         * Adds a pattern
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        with: <p_1 extends Pattern<a>>(pattern: p_1, expr: Fun<ListUpperBound<a, InvertPattern<p_1>>, unknown>) => any;
        /**
         * Adds a pattern with an additional predicate
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withWhen: <p_2 extends Pattern<a>>(pattern: p_2, when: Fun<ListUpperBound<a, InvertPattern<p_2>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_2>>, unknown>) => any;
        /**
         * Adds a negated pattern. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        withNot: <p_3 extends Pattern<a>>(pattern: p_3, expr: Fun<ListLowerBound<a, InvertPattern<p_3>>, unknown>) => any;
        withNotWhen: <p_4 extends Pattern<a>>(pattern: p_4, when: Fun<ListLowerBound<a, InvertPattern<p_4>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_4>>, unknown>) => any;
        /**
         * Sets the faalback value for when no pattern matches
         * @param otherwise   a function to create the result
         */
        otherwise: (otherwise: () => unknown) => any;
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: () => unknown;
    };
    /**
     * Sets the faalback value for when no pattern matches
     * @param otherwise   a function to create the result
     */
    otherwise: (otherwise: () => b) => {
        /**
         * Adds a pattern
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        with: <p_1 extends Pattern<a>>(pattern: p_1, expr: Fun<ListUpperBound<a, InvertPattern<p_1>>, unknown>) => any;
        /**
         * Adds a pattern with an additional predicate
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withWhen: <p_2 extends Pattern<a>>(pattern: p_2, when: Fun<ListUpperBound<a, InvertPattern<p_2>>, boolean>, expr: Fun<ListUpperBound<a, InvertPattern<p_2>>, unknown>) => any;
        /**
         * Adds a negated pattern. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        withNot: <p_3 extends Pattern<a>>(pattern: p_3, expr: Fun<ListLowerBound<a, InvertPattern<p_3>>, unknown>) => any;
        /**
         * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withNotWhen: <p_4 extends Pattern<a>>(pattern: p_4, when: Fun<ListLowerBound<a, InvertPattern<p_4>>, boolean>, expr: Fun<ListLowerBound<a, InvertPattern<p_4>>, unknown>) => any;
        otherwise: (otherwise: () => unknown) => any;
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: () => unknown;
    };
    /**
     * Runs the match and return the result of the first matching pattern
     */
    run: () => b;
};
