"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Constructs the builder API
 * @param value       the input value
 * @param otherwise   optional - a function that creates the fallback value
 * @param patterns    optional - an array with the patterns
 */
exports.match = function (value) { return builder(value)(function () { return undefined; }, []); };
var builder = function (value) { return function (otherwise, patterns) {
    if (otherwise === void 0) { otherwise = function () { return null; }; }
    if (patterns === void 0) { patterns = []; }
    return ({
        /**
         * Adds a pattern
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        with: function (pattern, expr) { return builder(value)(otherwise, __spreadArrays(patterns, [[pattern, function () { return true; }, expr, 'default']])); },
        /**
         * Adds a pattern with an additional predicate
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withWhen: function (pattern, when, expr) { return builder(value)(otherwise, __spreadArrays(patterns, [[pattern, when, expr, 'default']])); },
        /**
         * Adds a negated pattern. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param expr      the function to create the result with
         */
        withNot: function (pattern, expr) { return builder(value)(otherwise, __spreadArrays(patterns, [[pattern, function () { return true; }, expr, 'negated']])); },
        /**
         * Adds a negated pattern with an additional predicate. Negated patterns don't work with `any` as input
         * @param pattern   the pattern to test with
         * @param when      a predicate that has to be true, outside the mathing of the pattern
         * @param expr      the function to create the result with
         */
        withNotWhen: function (pattern, when, expr) { return builder(value)(otherwise, __spreadArrays(patterns, [[pattern, when, expr, 'negated']])); },
        /**
         * Sets the faalback value for when no pattern matches
         * @param otherwise   a function to create the result
         */
        otherwise: function (otherwise) { return builder(value)(otherwise, patterns); },
        /**
         * Runs the match and return the result of the first matching pattern
         */
        run: function () {
            var p = patterns.find(function (p) {
                if (p[1](value) == false)
                    return false;
                if (p[3] == 'default')
                    return match_pattern(value, p[0]);
                return !match_pattern(value, p[0]);
            });
            if (p == undefined)
                return otherwise();
            return p[2](value);
        }
    });
}; };
var match_pattern = function (value, pattern) {
    if (pattern === String)
        return typeof (value) == 'string';
    if (pattern === Boolean)
        return typeof (value) == 'boolean';
    if (pattern === Number)
        return typeof (value) == 'number' && Number.isNaN(value) == false;
    if (Array.isArray(pattern)) {
        if (!Array.isArray(value))
            return false;
        return value.every(function (v) { return match_pattern(v, pattern[0]); });
    }
    if (typeof (value) != 'object')
        return value === pattern;
    return Object.keys(pattern).every(function (k) { return pattern[k] == undefined ? false : match_pattern(value[k], pattern[k]); });
};
//# sourceMappingURL=main.js.map